CONVERT_JSON_TO_YAML = {
	"type": "post",
	"endpoint": "/convertJSONtoYAML",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
CONVERT_RANGE_FROM_TO = {
	"type": "get",
	"endpoint": "/convertRangeFromTo",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
CONVERT_TEXT_FILE_FROM_TO = {
	"type": "get",
	"endpoint": "/convertTextFileFromTo",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
CONVERT_YAML_TO_JSON = {
	"type": "post",
	"endpoint": "/convertYAMLtoJSON",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
COPY_FILE = {
	"type": "get",
	"endpoint": "/copyFile",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_FILE = {
	"type": "get",
	"endpoint": "/getFile",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

EXECUTE_GENERIC = {
	"type": "post",
	"endpoint": "/executeGeneric",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_CONFIG = {
	"type": "get",
	"endpoint": "/config",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
CREATE_JSON_FROM_TEXT = {
	"type": "get",
	"endpoint": "/createJSONfromTEXT",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_CONTAINER_LOGS = {
	"type": "get",
	"endpoint": "/getContainerLogs",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_REST_GENERIC = {
	"type": "post",
	"endpoint": "/getRestGeneric",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}