SEND_GET = {
	"type": "get",
	"endpoint": "/sendGet",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_REST = {
	"type": "get",
	"endpoint": "/getRest",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_CASSANDRA = {
	"type": "get",
	"endpoint": "/getCassandra",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_CASSANDRA_VERSION = {
	"type": "get",
	"endpoint": "/getCassandraVersion",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_FILE = {
	"type": "get",
	"endpoint": "/getFile",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_FILE_LIST = {
	"type": "get",
	"endpoint": "/getFileList",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_IP = {
	"type": "get",
	"endpoint": "/getIP",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_REST = {
	"type": "get",
	"endpoint": "/getREST",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
LIST_TO_MATRIX = {
	"type": "get",
	"endpoint": "/listToMatrix",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
MATRIX_TO_LIST = {
	"type": "get",
	"endpoint": "/matrixToList",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PULL_KAFKA_TOPIC = {
	"type": "get",
	"endpoint": "/pullKafkaTopic",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PUSH_KAFKA_TOPICS = {
	"type": "get",
	"endpoint": "/pushKafkaTopics",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
UPLOAD_FILE = {
	"type": "post",
	"endpoint": "/upload",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

FILE_DATABASE_IMPORT = {
	"type": "get",
	"endpoint": "/fileDatabaseImport",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_PROPERTY = {
	"type": "get",
	"endpoint": "/getProperty",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

UPDATE_PROPERTIES = {
	"type": "post",
	"endpoint": "/updateProperties",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

UPDATE_PROPERTIES_KEY = {
	"type": "get",
	"endpoint": "/updatePropertiesKey",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

REFRESH = {
	"type": "get",
	"endpoint": "/refresh",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_SPEND_PERSONALITY = {
	"type": "get",
	"endpoint": "/getSpendingPersonality",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}


GET_FINANCIAL_WELLNESS = {
	"type": "get",
	"endpoint": "/getFinancialWellnessScore",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

PUT_FINANCIAL_WELLNESS = {
	"type": "put",
	"endpoint": "/getFinancialWellnessScore",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

SQL_CASSANDRA = {
	"type": "get",
	"endpoint": "/sqlCassandra",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

CLOSE_CASSANDRA = {
	"type": "get",
	"endpoint": "/closeCassandra",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

TEST_KAFKA_KERBEROS = {
	"type": "get",
	"endpoint": "/testKafkaKerberos",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

ESTORE_RECOMMENDATIONS = {
	"type": "get",
	"endpoint": "/estoreRecommendations",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

PUT_ESTORE_RECOMMENDATIONS = {
	"type": "put",
	"endpoint": "/estoreRecommendations",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

CONNECT_U = {
	"type": "get",
	"endpoint": "/connectU",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

PUT_CONNECT_U = {
	"type": "put",
	"endpoint": "/connectU",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

JUST_FOR_YOU = {
	"type": "get",
	"endpoint": "/justforyou",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

PUT_JUST_FOR_YOU = {
	"type": "put",
	"endpoint": "/justforyou",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

ESTORE_RECOMMENDER_NON_GSM = {
	"type": "get",
	"endpoint": "/estoreRecommenderNonGSM",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

PUT_ESTORE_RECOMMENDER_NON_GSM = {
	"type": "put",
	"endpoint": "/estoreRecommenderNonGSM",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GIFT_RECOMMENDATIONS_FREE = {
	"type": "get",
	"endpoint": "/giftRecommendationsFree",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GIFT_RECOMMENDATIONS_PURCHASED = {
	"type": "get",
	"endpoint": "/giftRecommendationsPurchased",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GIFT_RECOMMENDATIONS = {
	"type": "put",
	"endpoint": "/giftRecommendations",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

BEHAVIOR_RECOMMENDER = {
	"type": "get",
	"endpoint": "/behaviorRecommender",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

PUT_BEHAVIOR_RECOMMENDER = {
	"type": "put",
	"endpoint": "/behaviorRecommender",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}


		