# Worker: Google
GET_SECURITY = {
	"type": "get",
	"endpoint": "/getSecurity",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}

GET_SENTIMENT = {
	"type": "get",
	"endpoint": "/getSentiment",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}