GET_TIMELINE = {
	"type": "get",
	"endpoint": "/getTimeline",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PROCESS_BASKET = {
	"type": "get",
	"endpoint": "/processBasket",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PROCESS_DIRECTED_GRAPH = {
	"type": "get",
	"endpoint": "/processDirectedGraph",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PROCESS_ECOGENETIC_NETWORK = {
	"type": "get",
	"endpoint": "/processEcogeneticNetwork",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PROCESS_APRIORI = {
	"type": "get",
	"endpoint": "/processApriori",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GENERATE_FORECAST = {
	"type": "get",
	"endpoint": "/generateForecast",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PROCESS_DIRECTED_GRAPH2 = {
	"type": "get",
	"endpoint": "/processDirectedGraph2",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GENERATE_COX_PH_DATA = {
	"type": "get",
	"endpoint": "/generateCoxPhData",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PROCESS_ARIMA_FORECAST = {
	"type": "post",
	"endpoint": "/processArimaForecast",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}



LIST_PULSE_RESPONDER_DYNAMIC = {
	"type": "get",
	"endpoint": "/listPulseResponderDynamic",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
DELETE_PULSE_RESPONDER_DYNAMIC = {
	"type": "get",
	"endpoint": "/deletePulseResponderDynamic",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_PULSE_RESPONDER_MESSAGES = {
	"type": "get",
	"endpoint": "/getPulseResponderMessages",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_PULSE_RESPONDER_OPTIONS = {
	"type": "get",
	"endpoint": "/getPulseResponderOptions",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_PULSE_RESPONDER_PROFILE = {
	"type": "get",
	"endpoint": "/getPulseResponderProfile",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
UPDATE_CLIENT_PULSE_RESPONDER = {
	"type": "post",
	"endpoint": "/updateClientPulseResponder",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
SAVE_PULSE_RESPONDER_DYNAMIC = {
	"type": "post",
	"endpoint": "/savePulseResponderDynamic",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
SAVE_PULSE_RESPONDER_PROFILE = {
	"type": "post",
	"endpoint": "/savePulseResponderProfile",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}