BUILD_MODEL = {
	"type": "get",
	"endpoint": "/buildModel",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
CANCEL_JOB = {
	"type": "get",
	"endpoint": "/cancelJob",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
DELETE_FRAME = {
	"type": "delete",
	"endpoint": "/deleteFrame",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
DOWNLOAD_MODEL_MOJO = {
	"type": "get",
	"endpoint": "/downloadModelMojo",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_AUTO_ML_MODEL = {
	"type": "get",
	"endpoint": "/getAutoMLmodel",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
GET_FRAME = {
	"type": "get",
	"endpoint": "/getFrame",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_FRAME_COLUMNS = {
	"type": "get",
	"endpoint": "/getFrameColumns",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_MODEL = {
	"type": "get",
	"endpoint": "/getModel",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
GET_MODEL_STATS = {
	"type": "get",
	"endpoint": "/getModelStats",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
MODEL_GRIDS = {
	"type": "get",
	"endpoint": "/modelGrids",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
PREDICTION_FRAMES = {
	"type": "get",
	"endpoint": "/predictionFrames",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
PREDICTION_JOBS = {
	"type": "get",
	"endpoint": "/predictionJobs",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
PREDICTION_MODELS = {
	"type": "get",
	"endpoint": "/predictionModels",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
PROCESS_FILE_TO_FRAME_IMPORT = {
	"type": "get",
	"endpoint": "/processFileToFrameImport",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
PROCESS_TO_FRAME_PARSE = {
	"type": "post",
	"endpoint": "/processToFrameParse",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
SPLIT_FRAME = {
	"type": "get",
	"endpoint": "/splitFrame",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
GENERATE_MODEL_DETAIL = {
	"type": "get",
	"endpoint": "/generateModelDetail",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
DOWNLOAD_MODEL = {
	"type": "get",
	"endpoint": "/downloadModel",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
IMPORT_SQL_TABLE = {
	"type": "post",
	"endpoint": "/importSQLTable",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
CHANGE_TO_ENUM = {
	"type": "get",
	"endpoint": "/changeToEnum",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
GET_FRAME_COLUMN_SUMMARY = {
	"type": "get",
	"endpoint": "/getFrameColumnSummary",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
EXPORT_FRAME = {
	"type": "get",
	"endpoint": "/exportFrame",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}