GET_SPENDING_PERSONALITY = {
	"type": "get",
	"endpoint": "/getSpendingPersonality",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PUT_SPENDING_PERSONALITY = {
	"type": "put",
	"endpoint": "/getSpendingPersonality",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
MODEL_DETAIL = {
	"type": "get",
	"endpoint": "/modelDetail",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_OFFER_RECOMMENDATIONS = {
	"type": "get",
	"endpoint": "/offerRecommendations",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PUT_OFFER_RECOMMENDATIONS = {
	"type": "put",
	"endpoint": "/offerRecommendations",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GET_PERSONALITY_RECOMMENDER = {
	"type": "get",
	"endpoint": "/personalityRecommender",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PUT_PERSONALITY_RECOMMENDER = {
	"type": "put",
	"endpoint": "/personalityRecommender",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PREDICTOR_RESPONSE_PRELOAD = {
	"type": "get",
	"endpoint": "/predictorResponsePreLoad",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PREDICTOR_RESPONSE_PRELOAD_KAFKA = {
	"type": "get",
	"endpoint": "/predictorResponsePreLoadKafKa",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
REFRESH = {
	"type": "get",
	"endpoint": "/refresh",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
RUN_MODEL_MOJO = {
	"type": "get",
	"endpoint": "/runModelMojo",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
INVOCATIONS = {
	"type": "post",
	"endpoint": "/invocations",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
RESPONSE = {
	"type": "post",
	"endpoint": "/response",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
PROCESS_BATCH = {
	"type": "post",
	"endpoint": "/processBatch",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}
GENERATE_KEY = {
	"type": "get",
	"endpoint": "/generateKey",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"	
}
GENERATE_CLASS = {
	"type": "post",
	"endpoint": "/generateClass",
	"call_message": "{type} {endpoint}",
	"error_message": "{type} {endpoint} {response_code}"
}